@@ -1,4 +1,5 @@
 from urllib.request import urlopen
+from urllib.error import HTTPError
 import json
 import pandas as pd
 
@@ -22,8 +23,12 @@ def available_companies(api_key):
         Data with the ticker as the index and the company name, price and
         stock exchange in the columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/stock/list?apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/stock/list?apikey={api_key}")
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
@@ -54,8 +59,12 @@ def profile(ticker, api_key):
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/profile/" + ticker + "?apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/profile/{ticker}?apikey={api_key}")
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
@@ -84,8 +93,12 @@ def quote(ticker, api_key):
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/quote/" + ticker + "?apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/quote/{ticker}?apikey={api_key}")
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
@@ -116,10 +129,14 @@ def enterprise(ticker, api_key, period="annual"):
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/enterprise-values/" +
-                       ticker + "?period=" + period + "&apikey=" + api_key)
-    data = response.read().decode("utf-8")
-    data_json = json.loads(data)
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/enterprise-values/{ticker}"
+                           f"?period={period}&apikey={api_key}")
+        data = response.read().decode("utf-8")
+        data_json = json.loads(data)
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data_json:
         raise ValueError(data_json['Error Message'])
@@ -155,15 +172,21 @@ def rating(ticker, api_key):
      data (dataframe)
         Data with variables in rows and the period in columns..
      """
-    response = urlopen("https://financialmodelingprep.com/api/v3/company/rating/" +
-                       ticker + "?apikey=" + api_key)
-    data = response.read().decode("utf-8")
-    data_json = json.loads(data)
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/historical-rating/{ticker}?apikey={api_key}")
+        data = response.read().decode("utf-8")
+        data_json = json.loads(data)
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data_json:
         raise ValueError(data_json['Error Message'])
 
-    data_formatted = pd.DataFrame(data_json["ratingDetails"]).T
+    for value in data_json:
+        del value['symbol']
+
+    data_formatted = pd.DataFrame(data_json).set_index('date')
 
     return data_formatted
 
@@ -189,9 +212,13 @@ def discounted_cash_flow(ticker, api_key, period="annual"):
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/discounted-cash-flow/" +
-                       ticker + "?period=" + period + "&apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/discounted-cash-flow/{ticker}"
+                           f"?period={period}&apikey={api_key}")
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
@@ -204,9 +231,13 @@ def discounted_cash_flow(ticker, api_key, period="annual"):
     except KeyError:
         pass
 
-    response = urlopen("https://financialmodelingprep.com/api/v3/historical-discounted-cash-flow/" +
-                       ticker + "?period=" + period + "&apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/historical-discounted-cash-flow/{ticker}"
+                           f"?period={period}&apikey={api_key}")
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
@@ -248,9 +279,12 @@ def earnings_calendar(api_key):
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/earning_calendar/" +
-                       "?apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    try:
+        response = urlopen(f"https://financialmodelingprep.com/api/v3/earning_calendar/?apikey={api_key}")
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
