@@ -1,9 +1,10 @@
 from urllib.request import urlopen
+from urllib.error import HTTPError
 import json
 import pandas as pd
 
 
-def income_statement(ticker, api_key, period="annual"):
+def income_statement(ticker, api_key, period="annual", as_reported=False):
     """
     Description
     ----
@@ -18,15 +19,26 @@ def income_statement(ticker, api_key, period="annual"):
         The API Key obtained from https://financialmodelingprep.com/developer/docs/
     period (string)
         Data period, this can be "annual" or "quarter".
+    as_reported (boolean)
+        Raw data without modifications.
 
     Output
     ----
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/income-statement/" +
-                       ticker + "?period=" + period + "&apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    if as_reported:
+        URL = (f"https://financialmodelingprep.com/api/v3/income-statement-as-reported/{ticker}"
+               f"?period={period}&apikey={api_key}")
+    else:
+        URL = f"https://financialmodelingprep.com/api/v3/income-statement/{ticker}?period={period}&apikey={api_key}"
+
+    try:
+        response = urlopen(URL)
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
@@ -45,7 +57,7 @@ def income_statement(ticker, api_key, period="annual"):
     return pd.DataFrame(data_formatted)
 
 
-def balance_sheet_statement(ticker, api_key, period="annual"):
+def balance_sheet_statement(ticker, api_key, period="annual", as_reported=False):
     """
     Description
     ----
@@ -60,15 +72,27 @@ def balance_sheet_statement(ticker, api_key, period="annual"):
         The API Key obtained from https://financialmodelingprep.com/developer/docs/
     period (string)
         Data period, this can be "annual" or "quarter".
+    as_reported (boolean)
+        Raw data without modifications.
 
     Output
     ----
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/balance-sheet-statement/" +
-                       ticker + "?period=" + period + "&apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    if as_reported:
+        URL = (f"https://financialmodelingprep.com/api/v3/balance-sheet-statement-as-reported/{ticker}"
+               f"?period={period}&apikey={api_key}")
+    else:
+        URL = (f"https://financialmodelingprep.com/api/v3/balance-sheet-statement/{ticker}"
+               f"?period={period}&apikey={api_key}")
+
+    try:
+        response = urlopen(URL)
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
@@ -87,7 +111,7 @@ def balance_sheet_statement(ticker, api_key, period="annual"):
     return pd.DataFrame(data_formatted)
 
 
-def cash_flow_statement(ticker, api_key, period="annual"):
+def cash_flow_statement(ticker, api_key, period="annual", as_reported=False):
     """
     Description
     ----
@@ -102,15 +126,27 @@ def cash_flow_statement(ticker, api_key, period="annual"):
         The API Key obtained from https://financialmodelingprep.com/developer/docs/
     period (string)
         Data period, this can be "annual" or "quarter".
+    as_reported (boolean)
+        Raw data without modifications.
 
     Output
     ----
     data (dataframe)
         Data with variables in rows and the period in columns.
     """
-    response = urlopen("https://financialmodelingprep.com/api/v3/cash-flow-statement/" +
-                       ticker + "?period=" + period + "&apikey=" + api_key)
-    data = json.loads(response.read().decode("utf-8"))
+    if as_reported:
+        URL = (f"https://financialmodelingprep.com/api/v3/cash-flow-statement-as-reported/{ticker}"
+               f"?period={period}&apikey={api_key}")
+    else:
+        URL = (f"https://financialmodelingprep.com/api/v3/cash-flow-statement/{ticker}"
+               f"?period={period}&apikey={api_key}")
+
+    try:
+        response = urlopen(URL)
+        data = json.loads(response.read().decode("utf-8"))
+    except HTTPError:
+        raise ValueError("This endpoint is only for premium members. Please visit the subscription page to upgrade the "
+                         "plan (Starter or higher) at https://financialmodelingprep.com/developer/docs/pricing")
 
     if 'Error Message' in data:
         raise ValueError(data['Error Message'])
